package com.compumark.gradle.plugin.ivy;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.ParseException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.jar.Attributes.Name;

import org.apache.ivy.Ivy;
import org.apache.ivy.core.module.id.ModuleRevisionId;
import org.apache.ivy.core.report.ArtifactDownloadReport;
import org.apache.ivy.core.report.ConfigurationResolveReport;
import org.apache.ivy.core.report.ResolveReport;
import org.apache.ivy.plugins.resolver.DependencyResolver;
import org.gradle.api.GradleException;
import org.gradle.api.Plugin;
import org.gradle.api.Project;
import org.gradle.api.Task;
import org.gradle.api.UnknownDomainObjectException;
import org.gradle.api.artifacts.ConfigurationContainer;
import org.gradle.api.artifacts.ExcludeRule;
import org.gradle.api.execution.TaskExecutionGraph;
import org.gradle.api.execution.TaskExecutionGraphListener;
import org.gradle.api.internal.file.collections.SimpleFileCollection;
import org.gradle.api.java.archives.Attributes;
import org.gradle.api.java.archives.Manifest;
import org.gradle.util.GUtil;

public class IvyPlugin implements Plugin<Project> {
	private final static String PUBLISH = "publish";
	private final static String RELEASE = "release";
	private final static String DEPENDS = "depends";
	
	private Project project;
	private Ivy ivy = Ivy.newInstance();
    
	@Override
	public void apply(Project project) {
		this.project = project;
		IvyPluginConvention ivyConvention = new IvyPluginConvention(project);
		project.getConvention().getPlugins().put("ivy", ivyConvention);
		
		try {
			System.out.println("##teamcity[progressMessage 'Configure from ivy']");
			IvyConfigure.configureIvy(project, ivy);
			configureRepositories();
			ResolveReport resolveReport = resolve();
			if (resolveReport != null) {
				if (resolveReport.hasError()) {
					throw new GradleException("Resolve failed");
				}
				configureConfigurations(resolveReport);
				configureDependencies(resolveReport);
				//configureExcludeRules(resolveReport);
				configureDependsTask(resolveReport);
				configurePublishTask(resolveReport);
				configureReleaseTask(resolveReport);
			}
		} 
		catch (Exception e) {
			throw new GradleException(e.getMessage(), e);
		}
		
		project.getGradle().getTaskGraph().addTaskExecutionGraphListener(new TaskExecutionGraphListener() {
			public void graphPopulated(TaskExecutionGraph graph) {		
				setManifestAttributes();
			}
        });
	}

	protected ResolveReport resolve() throws IOException, ParseException {
		if (project.hasProperty("ivyURL")) {
			return ivy.resolve((URL)project.property("ivyURL"));
		}
		File ivyFile = (File)project.property("ivyFile");
		if (ivyFile.exists()) {
			return ivy.resolve(ivyFile);
		}
		else {
			throw new GradleException(ivyFile.getAbsolutePath() + " does not exist");
		}				
	}		

	protected void setManifestAttributes() {
	     // Get TeamCity build number
		String buildNumber = System.getenv("BUILD_NUMBER");
		
		String status = "integration";
		List<Task> tasks = project.getGradle().getTaskGraph().getAllTasks();
		for (int i = 0 ; i < tasks.size() ; i++ ) {
			if (tasks.get(i).getName().equals(RELEASE)) {
				status = RELEASE;
				break;
			}
		}
		
        StringBuffer version = new StringBuffer();
		if (project.hasProperty("version")) {
			version.append(project.property("version").toString());
			if (buildNumber != null) {
				version.append("__");
				version.append(buildNumber);
			}
		}
		
		if (project.hasProperty("manifest")) {
	    	Manifest manifest = (Manifest)project.property("manifest");
	    	if (manifest != null) {
	    		Attributes attributes = manifest.getEffectiveManifest().getAttributes();
	    		attributes.put(Name.IMPLEMENTATION_TITLE.toString(), project.getName());
	    		attributes.put(Name.IMPLEMENTATION_VERSION.toString(), version.toString());
	    		attributes.put("Implementation-Status", status);
	    	}
		}
	}
	
	protected void configureRepositories() {
		for (@SuppressWarnings("rawtypes")
		Iterator iterator = ivy.getSettings().getResolvers().iterator(); iterator.hasNext();) {			
			DependencyResolver dependencyResolver = (DependencyResolver)iterator.next();
			project.getRepositories().add(dependencyResolver);
		}
	}	
			
	private void configureConfigurations(ResolveReport resolveReport) {
		ConfigurationContainer configurationContainer = project.getConfigurations();
		for (String conf : resolveReport.getConfigurations()) {
			try {
				configurationContainer.getByName(conf);
			} catch (UnknownDomainObjectException e) {
				configurationContainer.add(conf);
			}
		}
	}

	protected void configureDependencies(ResolveReport resolveReport) {
		for (String conf : resolveReport.getConfigurations()) {
			
			ConfigurationResolveReport report = resolveReport.getConfigurationReport(conf);
			for (@SuppressWarnings("unchecked")
			Iterator<ModuleRevisionId> iterator = report.getModuleRevisionIds().iterator(); iterator.hasNext();) {
				LinkedList<File> localFiles = new LinkedList<File>(); 
				ModuleRevisionId mrid = iterator.next();
				ArtifactDownloadReport[] downloadReports = report.getDownloadReports(mrid);
				for (int i = 0; i < downloadReports.length; i++) {
					localFiles.add(downloadReports[i].getLocalFile());
				}
				ModuleRevisionIdFileCollectionDependency branchFileCollectionDependency = new ModuleRevisionIdFileCollectionDependency(new SimpleFileCollection(localFiles), mrid);
				project.getDependencies().add(conf, branchFileCollectionDependency);
			}
		}
	}
		
	@SuppressWarnings("unchecked")
	protected void configureExcludeRules(ResolveReport resolveReport) {
		ConfigurationContainer configurationContainer = project.getConfigurations();
		for (org.apache.ivy.core.module.descriptor.ExcludeRule excludeRule : resolveReport.getModuleDescriptor().getAllExcludeRules()) {
			for (String conf : excludeRule.getConfigurations()) {
				String organisation = excludeRule.getId().getModuleId().getOrganisation();
				String moduleName = excludeRule.getId().getModuleId().getName();
				configurationContainer.getByName(conf).exclude(
					(Map<String, String>)GUtil.map(ExcludeRule.GROUP_KEY, organisation, ExcludeRule.MODULE_KEY, moduleName));
			}
		}
	}
		
	protected void configurePublishTask(ResolveReport resolveReport) {
		Publish publish = project.getTasks().add(PUBLISH, Publish.class);
        publish.setIvy(ivy);
        publish.setStatus(Publish.Status.INTEGRATION);
       	publish.getModuleDescriptors().add(resolveReport.getModuleDescriptor());
       	publish.setValidate(false);
        
        Task buildTask = project.getTasks().findByName("build");
        if (buildTask != null) {
        	publish.dependsOn(buildTask);
        }
    }

    protected void configureReleaseTask(ResolveReport resolveReport) {
		Publish release =  project.getTasks().add(RELEASE, Publish.class);
        release.setIvy(ivy);
        release.setStatus(Publish.Status.RELEASE);
       	release.getModuleDescriptors().add(resolveReport.getModuleDescriptor());
       	release.setValidate(true);
       	
        Task buildTask = project.getTasks().findByName("build");
        if (buildTask != null) {
        	release.dependsOn(buildTask);
        }
    }
    
    protected void configureDependsTask(ResolveReport resolveReport) {
		Depends depends = project.getTasks().add(DEPENDS, Depends.class);
        depends.setIvy(ivy);
       	depends.setResolveReport(resolveReport);

        Task classesTask = project.getTasks().findByName("classes");
        if (classesTask != null) {
        	classesTask.dependsOn(depends);
        }
    }
    
}